#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "app_uart.h"
#include "app_error.h"
#include "nrf_delay.h"
#include "nrf.h"
#include "nrf_gpio.h"
#include "nrf_drv_timer.h"
#include "twi_master.h"

#define ACC_ADR_R           0x31                                                                                /**< Accelerometer Adress in reading mode. */
#define ACC_ADR_W           0x30                                                                                /**< Accelerometer Adress in Writing mode. */
#define ACC_BURST           0x80                                                                                //Mask to set bit7 for Auto-inc mode (burst reads)
#define ACC_AUX_STATUS      0x07                                                                                //STATUS_AUX_REG
#define ACC_ADC1L           0x08                                                                                //OUT_ADC1_L
#define ACC_ADC1H           0x09                                                                                //OUT_ADC1_H
#define ACC_ADC2L           0x0A                                                                                //OUT_ADC2_L
#define ACC_ADC2H           0x0B                                                                                //OUT_ADC2_H
#define ACC_ADC3L           0x0C                                                                                //OUT_ADC3_L
#define ACC_ADC3H           0x0D                                                                                //OUT_ADC3_H
#define ACC_INT_CNT         0x0E                                                                                //INT_COUNTER_REG
#define ACC_WHO_AM_I        0x0F                                                                                //WHO_AM I
#define ACC_TEMP_CFG        0x1F                                                                                //TEMP_CFG_REG
#define ACC_CTRL1           0x20                                                                                //CTRL_REG1
#define ACC_CTRL2           0x21                                                                                //CTRL_REG2
#define ACC_CTRL3           0x22                                                                                //CTRL_REG3
#define ACC_CTRL4           0x23                                                                                //CTRL_REG4
#define ACC_CTRL5           0x24                                                                                //CTRL_REG5
#define ACC_CTRL6           0x25                                                                                //CTRL_REG6
#define ACC_REFDTCAP        0x26                                                                                //REFERENCE/DATACAPTURE
#define ACC_STATUS          0x27                                                                                //STATUS_REG2
#define ACC_OUTXL           0x28                                                                                //OUT_X_L
#define ACC_OUTXH           0x29                                                                                //OUT_X_H
#define ACC_OUTYL           0x2A                                                                                //OUT_Y_L
#define ACC_OUTYH           0x2B                                                                                //OUT_Y_H
#define ACC_OUTZL           0x2C                                                                                //OUT_Z_L
#define ACC_OUTZH           0x2D                                                                                //OUT_Z_H
#define ACC_FIFOCTRL        0x2E                                                                                //FIFO_CTRL_REG
#define ACC_FIFOSRC         0x2F                                                                                //FIFO_SRC_REG
#define ACC_INT1CFG         0x30                                                                                //INT1_CFG
#define ACC_INT1SRC         0x31                                                                                //INT1_SRC
#define ACC_INT1THS         0x32                                                                                //INT1_THS
#define ACC_INT1DRT         0x33                                                                                //INT1_DURATION
#define ACC_CLICCFG         0x38                                                                                //CLICK_CONFIG
#define ACC_CLICSRC         0x39                                                                                //CLICK_SRC
#define ACC_CLICTHS         0x3A                                                                                //CLICK_THS
#define ACC_TIMELMT         0x3B                                                                                //CLICK TIME_LIMIT
#define ACC_TIMELAT         0x3C                                                                                //CLICK TIME_LATENCY
#define ACC_TIMEWDW         0x3D                                                                                //CLICK TIME_WINDOW

#define ACC_SDBY            0x00                                                                                //Config CTRL_REG1 for Acc OFF
#define ALL_ON_LP_50H       0x4F                                                                                //Config CTRL_REG1 for Acc ON, all axis on, low power, 50Hz
#define ALL_ON_NM_200H      0x67                                                                                //Config CTRL_REG1 for Acc ON, all axis on, normal power, 1250Hz
#define HPF_ARIE            0xC9                                                                                //Config CTRL_REG2 for High-pass filter enabled with autoreset on interrupt event
#define AOI1_ON_INT1        0x40                                                                                //Config CTRL_REG3 for wakeup detection, interrupt driven to INT1 pad
#define WTM_ON_INT1         0x04                                                                                //Config CTRL_REG3 for watermark interrupt driven to INT1 pad

#define ACC_2G              0x08
#define ACC_4G              0x18                                                                                //Config CTRL_REG4 with +/-4G, Little endian
#define ACC_8G              0x28
#define ACC_16G             0x38                                                                                //Config CTRL_REG4 with +/-16G, Little endian
#define INT_LATCH           0x08                                                                                //Config CTRL_REG5 for Interrupt latched
#define FIFO_INT_LATCH      0x48                                                                                //Config CTRL_REG5 for Interrupt latched and FIFO Enabled
#define FIFO_INT_NOLATCH    0x40                                                                                //Config CTRL_REG5 for Interrupt not latched and FIFO Enabled

#define Sensitivity         2                                                                                        //Sensitivity when configured with +/-4G

#define LED_GREEN       27
#define ON_OFF          0

#define RX_PIN_NUMBER   8
#define TX_PIN_NUMBER   10

#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1                           /**< UART RX buffer size. */

#define BUFFER_SIZE                                     16

#define TEST_MODE                                       0

static bool saveToFlash = false;
uint8_t read_I2C[8] = {0};		        //I2C datas read
uint32_t * errorAddress = (uint32_t *)0x0003FFF4;

typedef struct dataAcc
{
    int16_t axisValue[3][32];
    uint8_t nbData;
} T_dataAcc;

/* Global variables */
char coord_buffer[] = "<000011112222>";


const nrf_drv_timer_t TIMER_PUBLISH = NRF_DRV_TIMER_INSTANCE(0);

void timer_publish_event_handler(nrf_timer_event_t event_type, void* p_context);
uint32_t accelero_init(void);
void manage_acc(void);

void timer_publish_event_handler(nrf_timer_event_t event_type, void* p_context)
{
    manage_acc();
}

void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{

}

uint32_t accelero_init(void)    
{
	uint8_t i;
    uint8_t data_reg5[]={ACC_CTRL5, 0x00};		//I2C datas to send to Reg5
	uint8_t data_I2C[]={ACC_CTRL1 + ACC_BURST, ALL_ON_NM_200H, 0x00, 0x00, ACC_16G, 0x00};		//I2C datas to send
	uint8_t check_I2C[]={0,0,0,0,0,0};	

    // Writing
	twi_master_transfer(ACC_ADR_W, data_I2C, sizeof(data_I2C), TWI_ISSUE_STOP);

    // Pause
    nrf_delay_ms(10);

    // Writing again REG5
    twi_master_transfer(ACC_ADR_W, data_reg5, 2, TWI_ISSUE_STOP);

    // Reading
	twi_master_transfer(ACC_ADR_W, data_I2C, 1, TWI_DONT_ISSUE_STOP);
	twi_master_transfer(ACC_ADR_R, check_I2C, 5, TWI_ISSUE_STOP);
        
	for(i = 0; i < 5; i++){
		if(data_I2C[i+1] != check_I2C[i]){            
            // write in flash
			return NRF_ERROR_INVALID_DATA;
		}
	}
	
	data_I2C[0] = ACC_FIFOCTRL + ACC_BURST;
    data_I2C[1] = 0x0;						//BYPASS
	twi_master_transfer(ACC_ADR_W, data_I2C, 2, TWI_ISSUE_STOP);
	twi_master_transfer(ACC_ADR_W, data_I2C, 1, TWI_DONT_ISSUE_STOP);
	twi_master_transfer(ACC_ADR_R, check_I2C, 2, TWI_ISSUE_STOP);		//read FIFO_SRC_REG at the same time
	

	if(data_I2C[1] != check_I2C[0]) {
      // write error in flash
	}

	return NRF_SUCCESS;
}


void manage_acc(void)
{
    bool res = false;
	static uint8_t      data_I2C[1];														//I2C datas to send
	
	static int16_t		data_x = 0;
    static int16_t		data_y = 0;
    static int16_t		data_z = 0;
	static char         coord_buff[16] = {0};

    data_I2C[0] = ACC_OUTXL + ACC_BURST;
    res = twi_master_transfer(ACC_ADR_W, data_I2C, 1, TWI_DONT_ISSUE_STOP);
    res = twi_master_transfer(ACC_ADR_R, &read_I2C[0], 2, TWI_ISSUE_STOP);		//read FIFO
    nrf_delay_ms(2);
    
    data_I2C[0] = ACC_OUTYL + ACC_BURST;
    res = twi_master_transfer(ACC_ADR_W, data_I2C, 1, TWI_DONT_ISSUE_STOP);
    res = twi_master_transfer(ACC_ADR_R, &read_I2C[2], 2, TWI_ISSUE_STOP);		//read FIFO
    nrf_delay_ms(2);
    
    data_I2C[0] = ACC_OUTZL + ACC_BURST;
    res = twi_master_transfer(ACC_ADR_W, data_I2C, 1, TWI_DONT_ISSUE_STOP);
    res = twi_master_transfer(ACC_ADR_R, &read_I2C[4], 2, TWI_ISSUE_STOP);		//read FIFO    
    nrf_delay_ms(2);
    
    if (!saveToFlash) {
         saveToFlash = true;
    }
}


static void flash_word_write(uint32_t * address, uint32_t value)
{
    // Turn on flash write enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Wen << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    *address = value;

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    // Turn off flash write enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }
}

static void flash_page_erase(uint32_t * page_address)
{
    // Turn on flash erase enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Een << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    // Erase page:
    NRF_NVMC->ERASEPAGE = (uint32_t)page_address;

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }

    // Turn off flash erase enable and wait until the NVMC is ready:
    NRF_NVMC->CONFIG = (NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos);

    while (NRF_NVMC->READY == NVMC_READY_READY_Busy)
    {
        // Do nothing.
    }
}

int main(void)
{   
    uint32_t time_ms;
    uint32_t err_code;
    uint32_t time_ticks;
    uint32_t * addr ;
    uint32_t flashCounter = 0;
    
    nrf_gpio_cfg_output(LED_GREEN);
    nrf_gpio_cfg_output(ON_OFF);
    nrf_gpio_cfg_output(4);
    
    nrf_gpio_pin_set(ON_OFF);
    nrf_gpio_pin_clear(LED_GREEN);
    nrf_gpio_pin_clear(4);
               
    twi_master_init();
    err_code = accelero_init();
    if (err_code != NRF_SUCCESS) {
        //save in flash
    }

    nrf_delay_ms(100);
    time_ms = 25;
    
   
    err_code = nrf_drv_timer_init(&TIMER_PUBLISH, NULL, timer_publish_event_handler);
    flash_word_write(errorAddress,(uint32_t)err_code);
    APP_ERROR_CHECK(err_code);
    time_ticks = nrf_drv_timer_ms_to_ticks(&TIMER_PUBLISH, time_ms);
    
    nrf_drv_timer_extended_compare(
         &TIMER_PUBLISH, NRF_TIMER_CC_CHANNEL0, time_ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);

    nrf_drv_timer_enable(&TIMER_PUBLISH);
    
    addr = (uint32_t *)(0x0003FFF0);
    while (true)
    {
        if ((*addr > 0x00001000) && (saveToFlash)){
            //flash_word_write(addr, (uint32_t)(read_I2C[2] + (read_I2C[3] << 8) + (read_I2C[4] << 16) + (read_I2C[5] << 24)));
            flash_word_write(addr, flashCounter);
            addr --;
            flashCounter++;
            saveToFlash = false;
        }
        if ((uint32_t)addr <= 0x00001000) {
            nrf_gpio_pin_set(LED_GREEN);
        }
    }
}


/** @} */
